import numpy as np


def lines_square(img, cutoff):
    nm = img > cutoff

    # make black borders
    nm[0, :] = False
    nm[-1, :] = False
    nm[:, 0] = False
    nm[:, -1] = False

    # break one vert corners
    one_vert = ~nm[:-1, :-1] & nm[1:, :-1] & nm[:-1, 1:] & ~nm[1:, 1:]
    nm[:-1, :-1] = np.where(one_vert, True, nm[:-1, :-1])
    one_vert = nm[:-1, :-1] & ~nm[1:, :-1] & ~nm[:-1, 1:] & nm[1:, 1:]
    nm[:-1, :-1] = np.where(one_vert, False, nm[:-1, :-1])

    # fill singles
    not_single = nm[:-2, 1:-1] & nm[2:, 1:-1] & nm[1:-1, :-2] & nm[1:-1, 2:]
    nm[1:-1, 1:-1] = np.where(not_single * (~nm[1:-1, 1:-1]), True, nm[1:-1, 1:-1])

    # edge: l[x,y] -> l[x,y-1]
    ylocs = np.argwhere(nm[1:, :] != nm[:-1, :])
    y_verts = [((l[0], l[1]), (l[0], l[1] - 1)) for l in ylocs]

    nmt = nm.copy()
    nmt[1:-1, :] = False
    ylocs = np.argwhere(nmt)
    y_verts.extend([((l[0], l[1]), (l[0], l[1] - 1)) for l in ylocs])

    # edge: l[x,y] -> l[x-1,y]
    xlocs = np.argwhere(nm[:, 1:] != nm[:, :-1])
    x_verts = [((l[0], l[1]), (l[0] - 1, l[1])) for l in xlocs]

    nmt = nm.copy()
    nmt[:, 1:-1] = False
    xlocs = np.argwhere(nmt)
    x_verts.extend([((l[0], l[1]), (l[0] - 1, l[1])) for l in xlocs])

    return x_verts + y_verts


def lines_marching(i_img, cutoff):
    # img = np.zeros((i_img.shape[0] + 2, i_img.shape[1] + 2), dtype=i_img.dtype)
    # img[1:-1, 1:-1] = i_img
    img = i_img

    nm = img > cutoff
    # nm = np.zeros((i_nm.shape[0] + 2, i_nm.shape[0] + 2), dtype=i_nm.dtype)
    # nm[1:-1, 1:-1] = i_nm

    # make black borders
    nm[0, :] = False
    nm[-1, :] = False
    nm[:, 0] = False
    nm[:, -1] = False

    zz = nm[:-1, :-1]
    oz = nm[1:, :-1]
    zo = nm[:-1, 1:]
    oo = nm[1:, 1:]

    verts = []

    ct_img = np.abs(cutoff - img)

    def _dif_y(v, pos):
        xpos = pos[0] + v[0]
        da = ct_img[xpos, pos[1]]
        db = ct_img[xpos, pos[1] + 1]
        td = da + db
        x, y = v[0], 0.0
        if td != 0:
            y = da * 0.999 / td
        return (x, y)

    def _dif_x(v, pos):
        ypos = pos[1] + v[1]
        da = ct_img[pos[0], ypos]
        db = ct_img[pos[0] + 1, ypos]
        td = da + db
        x, y = 0.0, v[1]
        if td != 0:
            x = da * 0.999 / td
        return (x, y)

    def _add(ia, da, check=None):
        ad0 = (da[0], da[1])
        ad1 = (da[2], da[3])

        locs = np.argwhere(ia)

        # make sure we are dealing with the same data we think we are
        # if check:
        #     for l in locs[:10]:
        #         ta = []
        #         ta.append(img[l[0], l[1]] > cutoff)
        #         ta.append(img[l[0] + 1, l[1]] > cutoff)
        #         ta.append(img[l[0], l[1] + 1] > cutoff)
        #         ta.append(img[l[0] + 1, l[1] + 1] > cutoff)
        #         assert sum(check[i] == v for i, v in enumerate(ta)) == 4

        f0 = _dif_y if ad0[0] == 0 or ad0[0] == 1 else _dif_x
        f1 = _dif_y if ad1[0] == 0 or ad1[0] == 1 else _dif_x

        for l in locs:
            m0x, m0y = f0(ad0, l)
            p0 = (l[0] + m0x, l[1] + m0y)

            m1x, m1y = f1(ad1, l)
            p1 = (l[0] + m1x, l[1] + m1y)

            verts.append((p0, p1))

    _add(zz & ~oz & ~zo & ~oo, [0, 0.5, 0.5, 0])
    _add(~zz & oz & ~zo & ~oo, [0.5, 0, 1, 0.5])
    _add(~zz & ~oz & zo & ~oo, [0, 0.5, 0.5, 1])
    _add(~zz & ~oz & ~zo & oo, [0.5, 1, 1, 0.5])

    _add(~zz & oz & zo & oo, [0, 0.5, 0.5, 0])
    _add(zz & ~oz & zo & oo, [0.5, 0, 1, 0.5])
    _add(zz & oz & ~zo & oo, [0, 0.5, 0.5, 1])
    _add(zz & oz & zo & ~oo, [0.5, 1, 1, 0.5])

    # _add(zz & oz & ~zo & ~oo, [0, 0.5, 1, 0.5], check=[1, 1, 0, 0])
    # _add(~zz & oz & ~zo & oo, [0.5, 0, 0.5, 1], check=[0, 1, 0, 1])
    # _add(~zz & ~oz & zo & oo, [0, 0.5, 1, 0.5], check=[0, 0, 1, 1])
    # _add(zz & ~oz & zo & ~oo, [0.5, 0, 0.5, 1], check=[1, 0, 1, 0])

    _add(zz & oz & ~zo & ~oo, [0, 0.5, 1, 0.5])
    _add(~zz & oz & ~zo & oo, [0.5, 0, 0.5, 1])

    _add(~zz & ~oz & zo & oo, [0, 0.5, 1, 0.5])
    _add(zz & ~oz & zo & ~oo, [0.5, 0, 0.5, 1])

    # \
    cross_a = ~zz & oz & zo & ~oo
    _add(cross_a, [0, 0.5, 0.5, 1])
    _add(cross_a, [0.5, 0, 1, 0.5])

    # /
    cross_b = zz & ~oz & ~zo & oo
    _add(cross_b, [0, 0.5, 0.5, 0])
    _add(cross_b, [0.5, 1, 1, 0.5])

    return verts
